package cerebral;

import prefuse.Constants;
import prefuse.Display;
import prefuse.Visualization;
import prefuse.action.ActionList;
import prefuse.action.RepaintAction;
import prefuse.action.animate.LocationAnimator;
import prefuse.data.Graph;
import prefuse.data.Table;
import prefuse.data.tuple.DefaultTupleSet;
import prefuse.data.tuple.TupleSet;
import prefuse.render.DefaultRendererFactory;
import prefuse.render.PolygonRenderer;
import prefuse.render.Renderer;
import prefuse.util.ColorLib;
import prefuse.visual.VisualGraph;
import prefuse.visual.VisualItem;
import prefuse.visual.expression.InGroupPredicate;
import cerebral.colors.ExpressionColorManager;
import cerebral.controls.BoundedPanControl;
import cerebral.controls.BoundedZoomControl;
import cerebral.controls.ClickToSelect;
import cerebral.controls.EdgeHighlightControl;
import cerebral.controls.GridDragger;
import cerebral.controls.NeighbourHighlightToGroupControl;
import cerebral.controls.RubberBandSelect;
import cerebral.controls.ToolTipLabel;
import cerebral.render.ExtendedShapeRenderer;
import cerebral.render.SmallLineDividerRenderer;
import cerebral.render.StraightEdgeRenderer;
import cerebral.sort.MyRenderSorter;
import cytoscape.CyNetwork;

public class SmallMultipleView extends Display {
    private static final long serialVersionUID = 3388992224966599415L;
    private String conditionName;
    private Visualization vis;
    private ExpressionColorManager eColorManager;
    
    public SmallMultipleView(String conditionName,
            ViewCoordinator viewCoordinator, 
            Cerebral cerebral,
            CyNetwork cyNetwork,
            Graph graph,
            TupleSet lineDividerTable,           
            EdgeBundler edgeBundler,
            Table edgeTable
            ) {
        this.conditionName = conditionName;
        vis = new Visualization();
        setVisualization(vis);
        
        VisualGraph vg = vis.addGraph(Cerebral.GRAPH, graph);
        vis.setInteractive(Cerebral.EDGES, null, true);
        vis.setInteractive(Cerebral.NODES, null, true);
        
        vis.add(Cerebral.LINE_DIVIDER, lineDividerTable);

        
        Table rubberBandTable = new Table();
        rubberBandTable.addColumn(VisualItem.POLYGON, float[].class);
        int r = rubberBandTable.addRow();
        rubberBandTable.set(r, VisualItem.POLYGON, new float[8]);
        
        vis.add(Cerebral.RUBBER_BAND, rubberBandTable);
        VisualItem rubberBand = (VisualItem) vis.getVisualGroup(Cerebral.RUBBER_BAND)
                .tuples().next();
        rubberBand.setStrokeColor(ColorLib.color(ColorLib.getColor(255, 0, 0)));
        rubberBand.setVisible(false);
        Renderer rubberBandRenderer = new PolygonRenderer(Constants.POLY_TYPE_LINE);
        
        
        ExtendedShapeRenderer nodeRenderer = new ExtendedShapeRenderer();
        
        nodeRenderer.setBaseSize(45);
        DefaultRendererFactory drf = new DefaultRendererFactory(nodeRenderer, new StraightEdgeRenderer());
        vis.setRendererFactory(drf);
        
        drf.add(new InGroupPredicate(Cerebral.LINE_DIVIDER), new SmallLineDividerRenderer(vis));
        drf.add(new InGroupPredicate(Cerebral.RUBBER_BAND), rubberBandRenderer);
        
       
        addControlListener(new BoundedPanControl(
                prefuse.controls.Control.MIDDLE_MOUSE_BUTTON, Cerebral.NODES, viewCoordinator));
        addControlListener(new BoundedZoomControl(Cerebral.NODES, viewCoordinator));
        addControlListener(new ClickToSelect(cerebral));
        addControlListener(new RubberBandSelect(cerebral, rubberBand));        
        addControlListener(new GridDragger(vis,vg.getNodes(),cerebral,edgeBundler));
        addControlListener(new ToolTipLabel(Cerebral.LABEL));
        
        
        TupleSet selectedItems = new DefaultTupleSet();
        vis.addFocusGroup(Cerebral.SELECTED, selectedItems);
        
        TupleSet hoverItems = new DefaultTupleSet();
        vis.addFocusGroup(Cerebral.NEIGHB_SELECT, hoverItems);
        addControlListener(new NeighbourHighlightToGroupControl("color", Cerebral.NEIGHB_SELECT, viewCoordinator));
        addControlListener(new EdgeHighlightControl("color", Cerebral.NEIGHB_SELECT,
                viewCoordinator));
        
        eColorManager = new ExpressionColorManager(cyNetwork, conditionName);
        eColorManager.createColorRules(vis, graph.getEdgeTable());
        vis.putAction("repaint", new RepaintAction()); 
        
        vis.run("color");
        vis.run("repaint");      
        
		ActionList animateNodes = new ActionList(1000);
		animateNodes.add(new LocationAnimator(Cerebral.NODES));
		animateNodes.add(new RepaintAction());	    
		vis.putAction("animateNodes", animateNodes);
		setItemSorter(new MyRenderSorter(edgeTable));

		setHighQuality(ControlPanel.instance().getQuality());
		viewCoordinator.addView(this);
        
		
}

    public ActionList getFillColor(){
        return eColorManager.getFillColor();
    }
   
    public String getConditonName(){
        return conditionName;
    }

    public ActionList getComparisonFill(SmallMultipleView select2) {
        return eColorManager.getComparisonFill(select2.getConditonName());
    }


  
}
