package cerebral.colors;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.plaf.metal.MetalButtonUI;

import prefuse.util.ColorLib;

public class ConstantRangeEditor extends JPanel implements
		PropertyChangeListener, FocusListener {

	private static final long serialVersionUID = 1L;
	private ConstantRange cr;
	private JFormattedTextField minVal, maxVal;
	private JButton colorButton;

	public ConstantRangeEditor() {
		setBackground(Color.white);
		setLayout(new FlowLayout(FlowLayout.LEFT));
		colorButton = new JButton("");
		colorButton.setUI(new MetalButtonUI());
		colorButton.setOpaque(true);
		colorButton.setPreferredSize(new Dimension(15, 15));

		colorButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (cr != null) {
					Color selectedColor = JColorChooser.showDialog(null,
							"Choose range color", ColorLib.getColor(cr.color));
					if (selectedColor != null) {
						cr.color = ColorLib.color(selectedColor);
						colorButton.setBackground(ColorLib.getColor(cr.color));
						firePropertyChange("color", null, null);
					}
				}
			}
		});
		add(colorButton);

		JLabel minLabel = new JLabel("Minimum");
		minLabel.setLabelFor(minVal);
		add(minLabel);

		minVal = new JFormattedTextField("#.####");
		minVal.setEditable(true);
		minVal.setValue(new Double(0.0));
		minVal.addPropertyChangeListener("value", this);
		minVal.addFocusListener(this);
		add(minVal);

		maxVal = new JFormattedTextField("#.####");
		maxVal.setEditable(true);
		maxVal.setValue(new Double(0.0));
		maxVal.addPropertyChangeListener("value", this);
		maxVal.addFocusListener(this);

		add(maxVal);
		JLabel maxLabel = new JLabel("Maximum");
		maxLabel.setLabelFor(maxVal);
		add(maxLabel);

	}

	public void setFocus() {
		minVal.requestFocus();
	}

	public void setConstantRange(ConstantRange crIn) {
		if (cr != null)
			cr.setBorder(BorderFactory.createEmptyBorder());
		cr = crIn;
		if (cr != null) {
			cr.setBorder(BorderFactory.createEtchedBorder());
			cr.setBackground(Color.yellow);
			colorButton.setBackground(ColorLib.getColor(cr.color));

			minVal.setValue(new Double(cr.getMin()));
			maxVal.setValue(new Double(cr.next().getMin()));
		}
	}

	public void commit() {
		commitMin();
		commitMax();
	}

	private void commitMin() {
		try {
			if (cr != null) {
				double newMin = Double.parseDouble(minVal.getText());
				if (newMin < cr.next().getMin()) {
					if (cr.prev() != null) {
						if (newMin < cr.prev().getMin()) {
							// expanded past previous. Disallow
							minVal.setForeground(Color.red);
							return;
						}
					}
					cr.setMin(newMin);
					minVal.setForeground(Color.black);
					firePropertyChange("min", null, null);
				} else {
					minVal.setForeground(Color.red);
				}
			}
		} catch (java.lang.NumberFormatException x) {
		}
	}

	private void commitMax() {
		try {

			if (cr != null) {
				double newMax = Double.parseDouble(maxVal.getText());
				if (newMax > cr.getMin()) {
					if (cr.next().next() != null) {
						if (newMax > cr.next().next().getMin()) {
							maxVal.setForeground(Color.red);
							return;
						}
					}
					cr.next().setMin(newMax);
					maxVal.setForeground(Color.black);
					firePropertyChange("max", null, null);
				} else {
					maxVal.setForeground(Color.red);
				}
			}
		} catch (java.lang.NumberFormatException x) {
		}

	}

	public void propertyChange(PropertyChangeEvent e) {
		if (e.getPropertyName() == "value") {
			if (e.getSource() == minVal) {
				commitMin();
			} else if (e.getSource() == maxVal) {
				commitMax();
			}
		}
	}

	public void focusGained(FocusEvent e) {
		if (e.getSource() instanceof JFormattedTextField) {
			JFormattedTextField text = (JFormattedTextField) e.getSource();
			String oldText = text.getText();
			text.setText("");
			text.setText(oldText);
			text.selectAll();
		}
	}

	public void focusLost(FocusEvent e) {
		// TODO Auto-generated method stub
	}
}
