package cerebral.colors;

import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.Border;

import prefuse.util.ColorLib;

public class ScaleEditor extends JPanel implements PropertyChangeListener,
        MouseListener, FocusListener {
    
    private static final long serialVersionUID = 1L;

    private ColorRangeComponent head;

    JButton addConstantMin, addLinearMin, addSigmoidMin;

    JButton addConstantMax, addLinearMax, addSigmoidMax;

    JButton decreaseAll;

    ColorLegend csLegend;

    JButton deleteRange;

    JFormattedTextField granularityText;

    JTextField stringRep;

    private double granularity = 2.0;

    JPanel rangeEditor = new JPanel();

    ConstantRangeEditor constantRangeEditor = new ConstantRangeEditor();

    LinearRangeEditor linearRangeEditor = new LinearRangeEditor();

    SigmoidalRangeEditor sigmoidalRangeEditor = new SigmoidalRangeEditor();
    
    Gallery gallery;

    private ColorRangeComponent selectedRange;

    private Font heading = new Font("Verdana", Font.PLAIN, 14);

    private Font regular = new Font("Verdana", Font.PLAIN, 12);

    private ScaleEditor outerThis = this;
    
    private String uneditedScale = "";

    public ScaleEditor(ColorRangeComponent headIn, String userScales) {
        head = headIn;
        gallery = new Gallery(userScales);

        setBackground(Color.white);
        setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 10;
        c.anchor = GridBagConstraints.LINE_START;
        c.insets = new Insets(0, 5, 5, 0);
        JLabel title = new JLabel("Edit current color scale.  Click on a segment to change range and color.");
        // title.setFont(heading);
        add(title, c);

        c.gridwidth = 1;
        c.gridy = 2;
        c.insets = new Insets(0, 10, 0, 5);
        addConstantMin = new JButton();
        addConstantMin.setIcon(new ImageIcon(getClass().getResource("/cerebral/colors/images/constantGreen.jpg")));
        addConstantMin.setBackground(Color.white);
        addConstantMin.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                deselectRange();
                double range = head.getRange();
                double min = head.getMin() - range;
                ConstantRange newRange = new ConstantRange(min, null, ColorLib
                        .rgb(26,150,65));
                head.lowerRange = newRange;
                newRange.higherRange = head;
                newRange.addMouseListener(outerThis);
                head = newRange;
                csLegend.setColorScale(head);
                constantRangeEditor.setConstantRange(newRange);
                CardLayout cl = (CardLayout) rangeEditor.getLayout();
                cl.show(rangeEditor, "constantRange");
                constantRangeEditor.setFocus();
                csLegend.update();
                selectRange(newRange);

            }
        });
        addConstantMin.setToolTipText("Add a solid color range to the low end of the scale");
        add(addConstantMin, c);
        

        c.gridy = 3;
        addLinearMin = new JButton();
        addLinearMin.setIcon(new ImageIcon(getClass().getResource("/cerebral/colors/images/linearGreen.jpg")));
        addLinearMin.setBackground(Color.white);
        
        addLinearMin.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                deselectRange();
                double range = head.getRange();
                double min = head.getMin() - range;
                LinearRange newRange = new LinearRange(min, null, ColorLib.rgb(
                		26,150,65), ColorLib.rgb(255,255,191));
                head.lowerRange = newRange;
                newRange.higherRange = head;
                newRange.addMouseListener(outerThis);
                head = newRange;
                csLegend.setColorScale(head);
                linearRangeEditor.setLinearRange(newRange);
                CardLayout cl = (CardLayout) rangeEditor.getLayout();
                cl.show(rangeEditor, "linearRange");
                csLegend.update();
                selectRange(newRange);
                linearRangeEditor.setFocus();
            }
        });

        addLinearMin.setToolTipText("Add a color gradient range to the low end of the scale");
        add(addLinearMin, c);

        /*
        c.gridy = 4;
        addSigmoidMin = new JButton();
        addSigmoidMin.setBackground(Color.white);        
        addSigmoidMin.setIcon(new ImageIcon(getClass().getResource("/cerebral/colors/images/sigmoidalGreen.jpg")));
        addSigmoidMin.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                deselectRange();
                double range = head.getRange();
                double min = head.getMin() - range;
                SigmoidalRange newRange = new SigmoidalRange(min, null,
                        ColorLib.rgb(26,150,65), 5.0, true);
                head.lowerRange = newRange;
                newRange.higherRange = head;
                newRange.addMouseListener(outerThis);
                head = newRange;
                csLegend.setColorScale(head);
                sigmoidalRangeEditor.setSigmoidalRange(newRange);
                CardLayout cl = (CardLayout) rangeEditor.getLayout();
                cl.show(rangeEditor, "sigmoidalRange");
                csLegend.update();
                selectRange(newRange);
                sigmoidalRangeEditor.setFocus();
            }
        });
        add(addSigmoidMin, c);
*/
        c.gridwidth = 8;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.weightx = 1.0;
        c.gridx = 1;
        c.gridy = 3;
        uneditedScale = ColorScaleBar.saveString(head);
        csLegend = new ColorLegend(head);       
        add(csLegend, c);

        ColorRangeComponent component = head;
        while (component.next() != null) {
            component.addMouseListener(this);
            component = (ColorRangeComponent) component.next();
        }

        c.gridwidth = 1;
        c.weightx = 0;
        c.gridx = 9;
        c.gridy = 1;

        c.gridy = 2;
        addConstantMax = new JButton();
        addConstantMax.setBackground(Color.white);        
        addConstantMax.setIcon(new ImageIcon(getClass().getResource("/cerebral/colors/images/constant.jpg")));
        addConstantMax.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                deselectRange();
                ColorRange r = head;
                while (r.next().next() != null) {
                    r = r.next();
                }
                double range = r.getRange();
                double min = r.getMin() + range;
                TerminateRange tr = (TerminateRange) r.next();
                ConstantRange newRange = new ConstantRange(min, r, ColorLib
                        .rgb(215,25, 28));
                tr.lowerRange = newRange;
                newRange.higherRange = tr;
                tr.min = newRange.getMin() + range;
                newRange.addMouseListener(outerThis);
                csLegend.setColorScale(head);
                constantRangeEditor.setConstantRange(newRange);
                CardLayout cl = (CardLayout) rangeEditor.getLayout();
                cl.show(rangeEditor, "constantRange");
                csLegend.update();
                selectRange(newRange);
                constantRangeEditor.setFocus();
            }
        });
        addConstantMax.setToolTipText("Add a solid color range to the high end of the scale");
        
        add(addConstantMax, c);

        c.gridy = 3;
        addLinearMax = new JButton();
        addLinearMin.setBackground(Color.white);        
        addLinearMax.setIcon(new ImageIcon(getClass().getResource("/cerebral/colors/images/linear.jpg")));
        addLinearMax.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                deselectRange();
                ColorRange r = head;
                while (r.next().next() != null) {
                    r = r.next();
                }
                double range = r.getRange();
                double min = r.getMin() + range;
                TerminateRange tr = (TerminateRange) r.next();
                LinearRange newRange = new LinearRange(min, r, ColorLib.rgb(255,
                        255, 191), ColorLib.rgb(215,25, 28));
                tr.lowerRange = newRange;
                newRange.higherRange = tr;
                tr.min = newRange.getMin() + range;
                newRange.addMouseListener(outerThis);
                csLegend.setColorScale(head);
                linearRangeEditor.setLinearRange(newRange);
                CardLayout cl = (CardLayout) rangeEditor.getLayout();
                cl.show(rangeEditor, "linearRange");
                csLegend.update();
                selectRange(newRange);
                linearRangeEditor.setFocus();
            }
        });

        addLinearMax.setToolTipText("Add a color gradient range to the high end of the scale");

        add(addLinearMax, c);

        /*
        c.gridy = 4;
        addSigmoidMax = new JButton();
        addSigmoidMax.setBackground(Color.white);        
        addSigmoidMax.setIcon(new ImageIcon(getClass().getResource("/cerebral/colors/images/sigmoidal.jpg")));
        addSigmoidMax.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                deselectRange();
                ColorRange r = head;
                while (r.next().next() != null) {
                    r = r.next();
                }
                double range = r.getRange();
                double min = r.getMin() + range;
                TerminateRange tr = (TerminateRange) r.next();
                SigmoidalRange newRange = new SigmoidalRange(min, r, ColorLib
                        .rgb(255,0, 0), 5.0, false);
                tr.lowerRange = newRange;
                newRange.higherRange = tr;
                tr.min = newRange.getMin() + range;
                newRange.addMouseListener(outerThis);
                csLegend.setColorScale(head);
                sigmoidalRangeEditor.setSigmoidalRange(newRange);
                CardLayout cl = (CardLayout) rangeEditor.getLayout();
                cl.show(rangeEditor, "sigmoidalRange");
                csLegend.update();
                selectRange(newRange);
                sigmoidalRangeEditor.setFocus();
            }
        });
        add(addSigmoidMax, c);
*/
        
        c.gridy = 5;
        c.gridx = 2;
        c.insets = new Insets(0, 0, 0, 0);
        JButton increaseAll = new JButton("*");
        increaseAll.setBackground(Color.white);
        add(increaseAll, c);
        increaseAll.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                // alterRange(cs, granularity);
                scale(granularity);
                csLegend.update();
            }

        });
        c.gridx = 3;
        decreaseAll = new JButton("/");
        decreaseAll.setBackground(Color.white);
        add(decreaseAll, c);
        decreaseAll.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                scale(1 / granularity);
                // alterRange(cs, -1*granularity);
                csLegend.update();
            }
        });

        c.gridx = 4;
        JLabel allRanges = new JLabel(" ranges by ");
        // allRanges.setFont(regular);
        add(allRanges, c);

        granularityText = new JFormattedTextField("#.###");
        granularityText.setValue(new Double(granularity));
        granularityText.setEditable(true);
        granularityText.addPropertyChangeListener("value", this);
        c.gridx = 5;
        add(granularityText, c);

        c.gridx = 2;
        c.gridwidth = 4;
        c.gridy = 6;
        deleteRange = new JButton("Delete range");
        // deleteRange.setFont(regular);
        add(deleteRange, c);
        deleteRange.setBackground(Color.white);
        deleteRange.setEnabled(false);
        deleteRange.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (selectedRange == head) {
                    head = (ColorRangeComponent) head.next();
                    csLegend.setColorScale(head);
                } else {
                    ColorRangeComponent prev = (ColorRangeComponent) selectedRange
                            .prev();
                    prev.higherRange = selectedRange.next();
                    ColorRangeComponent next = (ColorRangeComponent) selectedRange
                            .next();
                    next.lowerRange = prev;
                }
                deselectRange();
                CardLayout cl = (CardLayout) rangeEditor.getLayout();
                cl.show(rangeEditor, "blank");
                csLegend.update();
                deleteRange.setEnabled(false);
            }
        });
        c.gridy = 7;
        //Textbox to get the string representation of the color scale.  Useful for creating
        //galleries but ugly enought to hide from the user.
   
     //   stringRep = new JTextField();
    //    add(stringRep, c);
      //  stringRep.addFocusListener(this);

        c.gridwidth = 10;
        c.gridy = 10;
        c.gridx = 0;
        c.insets = new Insets(5, 1, 0, 1);
        rangeEditor.setMinimumSize(new Dimension(1, 200));
        rangeEditor.setLayout(new CardLayout());
        JPanel blankPanel = new JPanel();
        blankPanel.setBackground(Color.white);
        rangeEditor.add(blankPanel, "blank");
        rangeEditor.add(constantRangeEditor, "constantRange");
        rangeEditor.add(linearRangeEditor, "linearRange");
        rangeEditor.add(sigmoidalRangeEditor, "sigmoidalRange");
        CardLayout cl = (CardLayout) rangeEditor.getLayout();
        cl.show(rangeEditor, "blank");

        constantRangeEditor
                .addPropertyChangeListener(new PropertyChangeListener() {
                    public void propertyChange(PropertyChangeEvent evt) {
                        csLegend.update();
                    }
                });
        constantRangeEditor.addMouseListener(this);
        linearRangeEditor
                .addPropertyChangeListener(new PropertyChangeListener() {
                    public void propertyChange(PropertyChangeEvent evt) {
                        csLegend.update();
                    }
                });
        linearRangeEditor.addMouseListener(this);
        sigmoidalRangeEditor
                .addPropertyChangeListener(new PropertyChangeListener() {
                    public void propertyChange(PropertyChangeEvent evt) {
                        csLegend.update();
                    }
                });
        sigmoidalRangeEditor.addMouseListener(this);

        c.weightx = 1.0;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.LINE_START;
        add(rangeEditor, c);
        addMouseListener(this);

        c.gridy = 20;
        c.gridx = 0;
        c.insets = new Insets(5, 5, 0, 5);
        c.fill = GridBagConstraints.BOTH;
        c.weighty = 1.0;
        c.weightx = 1.0;
        add(gallery, c);
        gallery.addPropertyChangeListener("scaleChosen", this);

    }

    private void scale(double scale) {
        ColorRangeComponent range = head;
        while (range.next() != null) {
            range.min = range.min * scale;
            range = (ColorRangeComponent) range.next();
        }
        range.min = range.min * scale;
        CardLayout cl = (CardLayout) rangeEditor.getLayout();
        cl.show(rangeEditor, "blank");
        deselectRange();
    }

    private void deselectRange() {

    	linearRangeEditor.commit();
    	linearRangeEditor.setLinearRange(null);
    	constantRangeEditor.commit();
    	constantRangeEditor.setConstantRange(null);
    	sigmoidalRangeEditor.commit();
    	sigmoidalRangeEditor.setSigmoidalRange(null);
        deleteRange.setEnabled(false);
        
        selectedRange = null;
        Border empty = BorderFactory.createEmptyBorder();
        ColorRangeComponent range = head;
        while (range.next() != null) {
            range.setBorder(empty);
            range = (ColorRangeComponent) range.next();
        }
    }

    public void propertyChange(PropertyChangeEvent e) {
        if (e.getPropertyName() == "value") {
            try {
                if (e.getSource() == granularityText) {
                    JFormattedTextField jtf = (JFormattedTextField) e
                            .getSource();
                    granularity = Double.parseDouble(jtf.getText());
                }
            } catch (java.lang.NumberFormatException x) {
            }
        }else if (e.getPropertyName() == "scaleChosen"){
            ColorRangeComponent component = (ColorRangeComponent) head;
            while (component.next() != null) {
                component.removeMouseListener(this);
                component = (ColorRangeComponent) component.next();
            }
            String newScale = (String) e.getNewValue();
            ColorRange newHead = ColorScaleBar.loadFromString(newScale);
            csLegend.setColorScale(newHead);
            head = (ColorRangeComponent) newHead;
            CardLayout cl = (CardLayout) rangeEditor.getLayout();
            cl.show(rangeEditor, "blank");
            validate();
            component = (ColorRangeComponent) head;
            while (component.next() != null) {
                component.addMouseListener(this);
                component = (ColorRangeComponent) component.next();
            }
            uneditedScale = newScale;
        }
    }

    private void selectRange(ColorRangeComponent selection) {
        selectedRange = selection;
        // Enable, unless it's the last range
        deleteRange.setEnabled(head.next().next() != null);
    }

    public void mouseClicked(MouseEvent e) {
        JComponent source = (JComponent) e.getSource();
        if (source instanceof ConstantRange) {
            deselectRange();
            constantRangeEditor.setConstantRange((ConstantRange) source);
            CardLayout cl = (CardLayout) rangeEditor.getLayout();
            cl.show(rangeEditor, "constantRange");
            selectRange((ColorRangeComponent) source);
            constantRangeEditor.setFocus();

        } else if (source instanceof LinearRange) {
            deselectRange();
            linearRangeEditor.setLinearRange((LinearRange) source);
            CardLayout cl = (CardLayout) rangeEditor.getLayout();
            cl.show(rangeEditor, "linearRange");
            selectRange((ColorRangeComponent) source);
            linearRangeEditor.setFocus();

        } else if (source instanceof SigmoidalRange) {
            deselectRange();
            sigmoidalRangeEditor.setSigmoidalRange((SigmoidalRange) source);
            CardLayout cl = (CardLayout) rangeEditor.getLayout();
            cl.show(rangeEditor, "sigmoidalRange");
            selectRange((ColorRangeComponent) source);
            sigmoidalRangeEditor.setFocus();
        } else if (source instanceof ConstantRangeEditor) {
        } else if (source instanceof LinearRangeEditor) {
        } else if (source instanceof SigmoidalRangeEditor) {
        } else if (source instanceof ScaleEditor) {
            deselectRange();
            CardLayout cl = (CardLayout) rangeEditor.getLayout();
            cl.show(rangeEditor, "blank");

        }
    }

    public void mouseEntered(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public void mouseExited(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public void mousePressed(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public void mouseReleased(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public void focusGained(FocusEvent e) {
        if (e.getSource() == stringRep) {
            stringRep.setText(csLegend.csb.saveString());
            stringRep.selectAll();
        }
        // TODO Auto-generated method stub

    }

    public void focusLost(FocusEvent e) {
        // TODO Auto-generated method stub

    }

    public boolean userEdited() {
        return uneditedScale.compareTo(ColorScaleBar.saveString(head))!=0;
    }

}
