package cerebral.colors;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.plaf.metal.MetalButtonUI;

import prefuse.util.ColorLib;

public class SigmoidalRangeEditor extends JPanel implements
		PropertyChangeListener, FocusListener {

	private static final long serialVersionUID = 1L;

	private SigmoidalRange sr;

	private JFormattedTextField minVal, maxVal;

	private JButton colorButton;
	private JSlider sigmoidalValSlider;

	public SigmoidalRangeEditor() {
		setBackground(Color.white);
		setLayout(new FlowLayout(FlowLayout.LEFT));
		colorButton = new JButton("");
		colorButton.setUI(new MetalButtonUI());
		colorButton.setOpaque(true);
		colorButton.setPreferredSize(new Dimension(15, 15));

		colorButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (sr != null) {
					Color selectedColor = JColorChooser.showDialog(null,
							"Choose hue of sigmoid color function", ColorLib
									.getColor(ColorLib.hsb(sr.hsb[0], 1.0f,
											1.0f)));
					if (selectedColor != null) {
						sr.setColor(ColorLib.color(selectedColor));
						colorButton.setBackground(ColorLib.getColor(ColorLib
								.hsb(sr.hsb[0], 1.0f, 1.0f)));
						firePropertyChange("color", null, null);
					}
				}
			}
		});
		add(colorButton);

		JLabel minLabel = new JLabel("Minimum");
		minLabel.setLabelFor(minVal);
		add(minLabel);

		minVal = new JFormattedTextField("#.####");
		minVal.setEditable(true);
		minVal.setValue(new Double(0.0));
		minVal.addPropertyChangeListener("value", this);
		minVal.addFocusListener(this);
		add(minVal);

		maxVal = new JFormattedTextField("#.####");
		maxVal.setEditable(true);
		maxVal.setValue(new Double(0.0));
		maxVal.addPropertyChangeListener("value", this);
		maxVal.addFocusListener(this);
		add(maxVal);
		JLabel maxLabel = new JLabel("Maximum");
		maxLabel.setLabelFor(maxVal);
		add(maxLabel);

		sigmoidalValSlider = new JSlider(JSlider.HORIZONTAL, 1, 10, 5);
		sigmoidalValSlider.setSnapToTicks(true);
		sigmoidalValSlider.setPaintTicks(true);
		sigmoidalValSlider.setMajorTickSpacing(1);
		sigmoidalValSlider.setBackground(Color.white);
		sigmoidalValSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				sr.s = source.getValue();
				firePropertyChange("s", null, null);

			}
		});
		add(sigmoidalValSlider);

		JButton flip = new JButton();
		flip.setIcon(new ImageIcon(getClass().getResource(
				"/cerebral/colors/images/darrowsmall.jpg")));
		flip.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				sr.left = !sr.left;
				firePropertyChange("left", null, null);
			}
		});
		add(flip);

	}

	public void setFocus() {
		minVal.requestFocus();
	}

	public void setSigmoidalRange(SigmoidalRange srIn) {
		if (sr != null)
			sr.setBorder(BorderFactory.createEmptyBorder());
		sr = srIn;
		if (sr != null) {
			sr.setBorder(BorderFactory.createEtchedBorder());
			sr.setBackground(Color.yellow);
			colorButton.setBackground(ColorLib.getColor(ColorLib.hsb(sr.hsb[0],
					1.0f, 1.0f)));

			minVal.setValue(new Double(sr.getMin()));
			maxVal.setValue(new Double(sr.next().getMin()));
			sigmoidalValSlider.setValue((int) sr.s);
		}
	}

	public void commit() {
		commitMin();
		commitMax();
	}

	private void commitMin() {
		try {
			if (sr != null) {
				double newMin = Double.parseDouble(minVal.getText());
				if (newMin < sr.next().getMin()) {
					if (sr.prev() != null) {
						if (newMin < sr.prev().getMin()) {
							// expanded past previous. Disallow
							minVal.setForeground(Color.red);
							return;
						}
					}
					sr.setMin(newMin);
					minVal.setForeground(Color.black);
					firePropertyChange("min", null, null);
				} else {
					minVal.setForeground(Color.red);
				}
			}
		} catch (java.lang.NumberFormatException x) {
		}
	}

	private void commitMax() {
		try {

			if (sr != null) {
				double newMax = Double.parseDouble(maxVal.getText());
				if (newMax > sr.getMin()) {
					if (sr.next().next() != null) {
						if (newMax > sr.next().next().getMin()) {
							maxVal.setForeground(Color.red);
							return;
						}
					}
					sr.next().setMin(newMax);
					maxVal.setForeground(Color.black);
					firePropertyChange("max", null, null);
				} else {
					maxVal.setForeground(Color.red);
				}
			}
		} catch (java.lang.NumberFormatException x) {
		}

	}

	public void propertyChange(PropertyChangeEvent e) {
		if (e.getPropertyName() == "value") {
			if (e.getSource() == minVal) {
				commitMin();
			} else if (e.getSource() == maxVal) {
				commitMax();
			}
		}
	}

	public void focusGained(FocusEvent e) {
		if (e.getSource() instanceof JFormattedTextField) {
			JFormattedTextField text = (JFormattedTextField) e.getSource();
			String oldText = text.getText();
			text.setText("");
			text.setText(oldText);
			text.selectAll();
		}
	}

	public void focusLost(FocusEvent e) {
		// TODO Auto-generated method stub
	}
}
